@@ -1,6 +1,6 @@
 /*
 
-Copyright © 2020-2021 The k3d Author(s)
+Copyright © 2020-2022 The k3d Author(s)
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,7 @@ import (
 
 	"github.com/docker/go-connections/nat"
 	"github.com/sirupsen/logrus"
+	"inet.af/netaddr"
 
 	"github.com/spf13/cobra"
 	"github.com/spf13/viper"
@@ -41,7 +42,7 @@ import (
 	cliconfig "github.com/rancher/k3d/v5/cmd/util/config"
 	k3dCluster "github.com/rancher/k3d/v5/pkg/client"
 	"github.com/rancher/k3d/v5/pkg/config"
-	conf "github.com/rancher/k3d/v5/pkg/config/v1alpha3"
+	conf "github.com/rancher/k3d/v5/pkg/config/v1alpha4"
 	l "github.com/rancher/k3d/v5/pkg/logger"
 	"github.com/rancher/k3d/v5/pkg/runtimes"
 	k3d "github.com/rancher/k3d/v5/pkg/types"
@@ -101,27 +102,11 @@ func NewCmdClusterCreate() *cobra.Command {
 			/*************************
 			 * Compute Configuration *
 			 *************************/
-			if cfgViper.GetString("apiversion") == "" {
-				cfgViper.Set("apiversion", config.DefaultConfigApiVersion)
-			}
-			if cfgViper.GetString("kind") == "" {
-				cfgViper.Set("kind", "Simple")
-			}
-			cfg, err := config.FromViper(cfgViper)
+			simpleCfg, err := config.SimpleConfigFromViper(cfgViper)
 			if err != nil {
 				l.Log().Fatalln(err)
 			}
 
-			if cfg.GetAPIVersion() != config.DefaultConfigApiVersion {
-				l.Log().Warnf("Default config apiVersion is '%s', but you're using '%s': consider migrating.", config.DefaultConfigApiVersion, cfg.GetAPIVersion())
-				cfg, err = config.Migrate(cfg, config.DefaultConfigApiVersion)
-				if err != nil {
-					l.Log().Fatalln(err)
-				}
-			}
-
-			simpleCfg := cfg.(conf.SimpleConfig)
-
 			l.Log().Debugf("========== Simple Config ==========\n%+v\n==========================\n", simpleCfg)
 
 			simpleCfg, err = applyCLIOverrides(simpleCfg)
@@ -266,6 +251,9 @@ func NewCmdClusterCreate() *cobra.Command {
 	cmd.Flags().String("registry-create", "", "Create a k3d-managed registry and connect it to the cluster (Format: `NAME[:HOST][:HOSTPORT]`\n - Example: `k3d cluster create --registry-create mycluster-registry:0.0.0.0:5432`")
 	_ = ppViper.BindPFlag("cli.registries.create", cmd.Flags().Lookup("registry-create"))
 
+	cmd.Flags().StringArray("host-alias", nil, "Add `ip:host[,host,...]` mappings")
+	_ = ppViper.BindPFlag("hostaliases", cmd.Flags().Lookup("host-alias"))
+
 	/* k3s */
 	cmd.Flags().StringArray("k3s-arg", nil, "Additional args passed to k3s command (Format: `ARG@NODEFILTER[;@NODEFILTER]`)\n - Example: `k3d cluster create --k3s-arg \"--disable=traefik@server:0\"")
 	_ = ppViper.BindPFlag("cli.k3sargs", cmd.Flags().Lookup("k3s-arg"))
@@ -327,6 +315,9 @@ func NewCmdClusterCreate() *cobra.Command {
 	cmd.Flags().String("agents-memory", "", "Memory limit imposed on the agents nodes [From docker]")
 	_ = cfgViper.BindPFlag("options.runtime.agentsmemory", cmd.Flags().Lookup("agents-memory"))
 
+	cmd.Flags().Bool("host-pid-mode", false, "Enable host pid mode of server(s) and agent(s)")
+	_ = cfgViper.BindPFlag("options.runtime.hostpidmode", cmd.Flags().Lookup("host-pid-mode"))
+
 	/* Image Importing */
 	cmd.Flags().Bool("no-image-volume", false, "Disable the creation of a volume for importing images")
 	_ = cfgViper.BindPFlag("options.k3d.disableimagevolume", cmd.Flags().Lookup("no-image-volume"))
@@ -590,7 +581,38 @@ func applyCLIOverrides(cfg conf.SimpleConfig) (conf.SimpleConfig, error) {
 			cfg.Registries.Create.Host = exposeAPI.Host
 			cfg.Registries.Create.HostPort = exposeAPI.Binding.HostPort
 		}
+	}
 
+	// --host-alias
+	hostAliasFlags := ppViper.GetStringSlice("hostaliases")
+	if len(hostAliasFlags) > 0 {
+		for _, ha := range hostAliasFlags {
+
+			// split on :
+			s := strings.Split(ha, ":")
+			if len(s) != 2 {
+				return cfg, fmt.Errorf("invalid format of host-alias %s (exactly one ':' allowed)", ha)
+			}
+
+			// validate IP
+			ip, err := netaddr.ParseIP(s[0])
+			if err != nil {
+				return cfg, fmt.Errorf("invalid IP '%s' in host-alias '%s': %w", s[0], ha, err)
+			}
+
+			// hostnames
+			hostnames := strings.Split(s[1], ",")
+			for _, hostname := range hostnames {
+				if err := k3dCluster.ValidateHostname(hostname); err != nil {
+					return cfg, fmt.Errorf("invalid hostname '%s' in host-alias '%s': %w", hostname, ha, err)
+				}
+			}
+
+			cfg.HostAliases = append(cfg.HostAliases, k3d.HostAlias{
+				IP:        ip.String(),
+				Hostnames: hostnames,
+			})
+		}
 	}
 
 	return cfg, nil
