@@ -1,5 +1,5 @@
 /*
-Copyright © 2020-2021 The k3d Author(s)
+Copyright © 2020-2022 The k3d Author(s)
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,8 +27,6 @@ import (
 	_ "embed"
 	"errors"
 	"fmt"
-	"io"
-	"os"
 	"sort"
 	"strconv"
 	"strings"
@@ -38,7 +36,7 @@ import (
 	"github.com/imdario/mergo"
 	copystruct "github.com/mitchellh/copystructure"
 	"github.com/rancher/k3d/v5/pkg/actions"
-	config "github.com/rancher/k3d/v5/pkg/config/v1alpha3"
+	config "github.com/rancher/k3d/v5/pkg/config/v1alpha4"
 	l "github.com/rancher/k3d/v5/pkg/logger"
 	"github.com/rancher/k3d/v5/pkg/runtimes"
 	k3drt "github.com/rancher/k3d/v5/pkg/runtimes"
@@ -74,6 +72,7 @@ func ClusterRun(ctx context.Context, runtime k3drt.Runtime, clusterConfig *confi
 	/*
 	 * Step 2: Pre-Start Configuration
 	 */
+	// Gather Environment information, e.g. the host gateway address
 	envInfo, err := GatherEnvironmentInfo(ctx, runtime, &clusterConfig.Cluster)
 	if err != nil {
 		return fmt.Errorf("failed to gather environment information used for cluster creation: %w", err)
@@ -88,6 +87,7 @@ func ClusterRun(ctx context.Context, runtime k3drt.Runtime, clusterConfig *confi
 		NodeHooks:       clusterConfig.ClusterCreateOpts.NodeHooks,
 		EnvironmentInfo: envInfo,
 		Intent:          k3d.IntentClusterCreate,
+		HostAliases:     clusterConfig.ClusterCreateOpts.HostAliases,
 	}); err != nil {
 		return fmt.Errorf("Failed Cluster Start: %+v", err)
 	}
@@ -309,9 +309,11 @@ func ClusterPrepImageVolume(ctx context.Context, runtime k3drt.Runtime, cluster
 	if err := runtime.CreateVolume(ctx, imageVolumeName, map[string]string{k3d.LabelClusterName: cluster.Name}); err != nil {
 		return fmt.Errorf("failed to create image volume '%s' for cluster '%s': %w", imageVolumeName, cluster.Name, err)
 	}
+	l.Log().Infof("Created image volume %s", imageVolumeName)
 
 	clusterCreateOpts.GlobalLabels[k3d.LabelImageVolume] = imageVolumeName
 	cluster.ImageVolume = imageVolumeName
+	cluster.Volumes = append(cluster.Volumes, imageVolumeName)
 
 	// attach volume to nodes
 	for _, node := range cluster.Nodes {
@@ -506,11 +508,13 @@ ClusterCreatOpts:
 	if !clusterCreateOpts.DisableLoadBalancer {
 		if cluster.ServerLoadBalancer == nil {
 			l.Log().Infof("No loadbalancer specified, creating a default one...")
-			lbNode, err := LoadbalancerPrepare(ctx, runtime, cluster, &k3d.LoadbalancerCreateOpts{Labels: clusterCreateOpts.GlobalLabels})
+			cluster.ServerLoadBalancer = k3d.NewLoadbalancer()
+			var err error
+			cluster.ServerLoadBalancer.Node, err = LoadbalancerPrepare(ctx, runtime, cluster, &k3d.LoadbalancerCreateOpts{Labels: clusterCreateOpts.GlobalLabels})
 			if err != nil {
 				return fmt.Errorf("failed to prepare loadbalancer: %w", err)
 			}
-			cluster.Nodes = append(cluster.Nodes, lbNode) // append lbNode to list of cluster nodes, so it will be considered during rollback
+			cluster.Nodes = append(cluster.Nodes, cluster.ServerLoadBalancer.Node) // append lbNode to list of cluster nodes, so it will be considered during rollback
 		}
 
 		if len(cluster.ServerLoadBalancer.Config.Ports) == 0 {
@@ -521,7 +525,11 @@ ClusterCreatOpts:
 			cluster.ServerLoadBalancer.Config = &lbConfig
 		}
 
-		cluster.ServerLoadBalancer.Node.RuntimeLabels = clusterCreateOpts.GlobalLabels
+		// ensure labels
+		cluster.ServerLoadBalancer.Node.FillRuntimeLabels()
+		for k, v := range clusterCreateOpts.GlobalLabels {
+			cluster.ServerLoadBalancer.Node.RuntimeLabels[k] = v
+		}
 
 		// prepare to write config to lb container
 		configyaml, err := yaml.Marshal(cluster.ServerLoadBalancer.Config)
@@ -541,6 +549,7 @@ ClusterCreatOpts:
 		}
 
 		cluster.ServerLoadBalancer.Node.HookActions = append(cluster.ServerLoadBalancer.Node.HookActions, writeLbConfigAction)
+		cluster.ServerLoadBalancer.Node.Restart = true
 
 		l.Log().Infof("Creating LoadBalancer '%s'", cluster.ServerLoadBalancer.Node.Name)
 		if err := NodeCreate(ctx, runtime, cluster.ServerLoadBalancer.Node, k3d.NodeCreateOpts{}); err != nil {
@@ -640,11 +649,12 @@ func ClusterDelete(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Clus
 		}
 	}
 
-	// delete image volume
-	if cluster.ImageVolume != "" {
-		l.Log().Infof("Deleting image volume '%s'", cluster.ImageVolume)
-		if err := runtime.DeleteVolume(ctx, cluster.ImageVolume); err != nil {
-			l.Log().Warningf("Failed to delete image volume '%s' of cluster '%s': Try to delete it manually", cluster.ImageVolume, cluster.Name)
+	// delete managed volumes attached to this cluster
+	l.Log().Infof("Deleting %d attached volumes...", len(cluster.Volumes))
+	for _, vol := range cluster.Volumes {
+		l.Log().Debugf("Deleting volume %s...", vol)
+		if err := runtime.DeleteVolume(ctx, vol); err != nil {
+			l.Log().Warningf("Failed to delete volume '%s' of cluster '%s': %v -> Try to delete it manually", cluster.ImageVolume, err, cluster.Name)
 		}
 	}
 
@@ -806,6 +816,12 @@ func ClusterGet(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Cluster
 		}
 	}
 
+	vols, err := runtime.GetVolumesByLabel(ctx, map[string]string{types.LabelClusterName: cluster.Name})
+	if err != nil {
+		return nil, err
+	}
+	cluster.Volumes = append(cluster.Volumes, vols...)
+
 	if err := populateClusterFieldsFromLabels(cluster); err != nil {
 		l.Log().Warnf("Failed to populate cluster fields from node labels: %v", err)
 	}
@@ -961,78 +977,101 @@ func ClusterStart(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Clust
 
 		/*** DNS ***/
 
-		// add host.k3d.internal record to /etc/hosts in all nodes
-		postStartErrgrp.Go(func() error {
-			return prepInjectHostIP(postStartErrgrpCtx, runtime, cluster, &clusterStartOpts)
-		})
+		// -> skip if hostnetwork mode
+		if cluster.Network.Name == "host" {
+			l.Log().Debugf("Not injecting hostAliases into /etc/hosts and CoreDNS as clusternetwork is 'host'")
+		} else {
+			// -> add hostAliases to /etc/hosts in all nodes
+			// --> inject host-gateway as host.k3d.internal
+			clusterStartOpts.HostAliases = append(clusterStartOpts.HostAliases, k3d.HostAlias{
+				IP:        clusterStartOpts.EnvironmentInfo.HostGateway.String(),
+				Hostnames: []string{"host.k3d.internal"},
+			})
 
-		if len(servers) > 0 {
-			postStartErrgrp.Go(func() error {
+			for _, node := range append(servers, agents...) {
+				currNode := node
+				postStartErrgrp.Go(func() error {
+					return NewHostAliasesInjectEtcHostsAction(runtime, clusterStartOpts.HostAliases).Run(postStartErrgrpCtx, currNode)
+				})
+			}
 
-				if cluster.Network.Name == "host" {
-					l.Log().Debugf("Not injecting host.k3d.internal into CoreDNS as clusternetwork is 'host'")
-					return nil
-				}
+			// -> inject hostAliases and network members into CoreDNS configmap
+			if len(servers) > 0 {
+				postStartErrgrp.Go(func() error {
 
-				hosts := fmt.Sprintf("%s %s\n", clusterStartOpts.EnvironmentInfo.HostGateway.String(), k3d.DefaultK3dInternalHostRecord)
+					hosts := ""
 
-				net, err := runtime.GetNetwork(ctx, &cluster.Network)
-				if err != nil {
-					return fmt.Errorf("failed to get cluster network %s to inject host records into CoreDNS: %w", cluster.Network.Name, err)
-				}
-				for _, member := range net.Members {
-					hosts += fmt.Sprintf("%s %s\n", member.IP.String(), member.Name)
-				}
+					// hosts: hostAliases (including host.k3d.internal)
+					for _, hostAlias := range clusterStartOpts.HostAliases {
+						hosts += fmt.Sprintf("%s %s\n", hostAlias.IP, strings.Join(hostAlias.Hostnames, " "))
+					}
 
-				l.Log().Infof("Injecting records for host.k3d.internal and for %d network members into CoreDNS configmap...", len(net.Members))
-				act := actions.RewriteFileAction{
-					Runtime: runtime,
-					Path:    "/var/lib/rancher/k3s/server/manifests/coredns.yaml",
-					Mode:    0744,
-					RewriteFunc: func(input []byte) ([]byte, error) {
-						split, err := util.SplitYAML(input)
-						if err != nil {
-							return nil, fmt.Errorf("error splitting yaml: %w", err)
-						}
+					// more hosts: network members ("neighbor" containers)
+					net, err := runtime.GetNetwork(postStartErrgrpCtx, &cluster.Network)
+					if err != nil {
+						return fmt.Errorf("failed to get cluster network %s to inject host records into CoreDNS: %w", cluster.Network.Name, err)
+					}
+					for _, member := range net.Members {
+						hosts += fmt.Sprintf("%s %s\n", member.IP.String(), member.Name)
+					}
 
-						var outputBuf bytes.Buffer
-						outputEncoder := yaml.NewEncoder(&outputBuf)
-
-						for _, d := range split {
-							var doc map[string]interface{}
-							if err := yaml.Unmarshal(d, &doc); err != nil {
-								return nil, err
+					// inject CoreDNS configmap
+					l.Log().Infof("Injecting records for hostAliases (incl. host.k3d.internal) and for %d network members into CoreDNS configmap...", len(net.Members))
+					act := actions.RewriteFileAction{
+						Runtime: runtime,
+						Path:    "/var/lib/rancher/k3s/server/manifests/coredns.yaml",
+						Mode:    0744,
+						RewriteFunc: func(input []byte) ([]byte, error) {
+							split, err := util.SplitYAML(input)
+							if err != nil {
+								return nil, fmt.Errorf("error splitting yaml: %w", err)
 							}
-							if kind, ok := doc["kind"]; ok {
-								if strings.ToLower(kind.(string)) == "configmap" {
-									configmapData := doc["data"].(map[interface{}]interface{})
-									configmapData["NodeHosts"] = hosts
+
+							var outputBuf bytes.Buffer
+							outputEncoder := yaml.NewEncoder(&outputBuf)
+
+							for _, d := range split {
+								var doc map[string]interface{}
+								if err := yaml.Unmarshal(d, &doc); err != nil {
+									return nil, err
+								}
+								if kind, ok := doc["kind"]; ok {
+									if strings.ToLower(kind.(string)) == "configmap" {
+										configmapData := doc["data"].(map[interface{}]interface{})
+										configmapData["NodeHosts"] = hosts
+									}
+								}
+								if err := outputEncoder.Encode(doc); err != nil {
+									return nil, err
 								}
 							}
-							if err := outputEncoder.Encode(doc); err != nil {
-								return nil, err
+							outputEncoder.Close()
+							return outputBuf.Bytes(), nil
+						},
+					}
+
+					// get the first server in the list and run action on it once it's ready for it
+					for _, n := range servers {
+
+						// do not try to run the action, if CoreDNS is disabled on K3s level
+						for _, flag := range n.Args {
+							if strings.HasPrefix(flag, "--disable") && strings.Contains(flag, "coredns") {
+								l.Log().Debugf("CoreDNS disabled in K3s via flag `%s`. Not trying to use it.", flag)
+								return nil
 							}
 						}
-						outputEncoder.Close()
-						return outputBuf.Bytes(), nil
-					},
-				}
-
-				// get the first server in the list and run action on it once it's ready for it
-				for _, n := range cluster.Nodes {
-					if n.Role == k3d.ServerRole {
 						ts, err := time.Parse("2006-01-02T15:04:05.999999999Z", n.State.Started)
 						if err != nil {
 							return err
 						}
-						if err := NodeWaitForLogMessage(ctx, runtime, n, "Cluster dns configmap", ts.Truncate(time.Second)); err != nil {
+						if err := NodeWaitForLogMessage(postStartErrgrpCtx, runtime, n, "Cluster dns configmap", ts.Truncate(time.Second)); err != nil {
 							return err
 						}
-						return act.Run(ctx, n)
+						return act.Run(postStartErrgrpCtx, n)
 					}
-				}
-				return nil
-			})
+					return nil
+				})
+			}
 		}
 
 		if err := postStartErrgrp.Wait(); err != nil {
@@ -1072,87 +1111,6 @@ func SortClusters(clusters []*k3d.Cluster) []*k3d.Cluster {
 	return clusters
 }
 
-// corednsAddHost adds a host entry to the CoreDNS configmap if it doesn't exist (a host entry is a single line of the form "IP HOST")
-func corednsAddHost(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Cluster, ip string, name string) error {
-	retries := 3
-	if v, ok := os.LookupEnv(k3d.K3dEnvDebugCorednsRetries); ok && v != "" {
-		l.Log().Debugf("Running with %s=%s", k3d.K3dEnvDebugCorednsRetries, v)
-		if r, err := strconv.Atoi(v); err == nil {
-			retries = r
-		} else {
-			return fmt.Errorf("Invalid value set for env var %s (%s): %w", k3d.K3dEnvDebugCorednsRetries, v, err)
-		}
-	}
-
-	// select any server node
-	var node *k3d.Node
-	for _, n := range cluster.Nodes {
-		if n.Role == k3d.ServerRole {
-			node = n
-		}
-	}
-
-	hostsEntry := fmt.Sprintf("%s %s", ip, name)
-	patchCmd := `patch=$(kubectl get cm coredns -n kube-system --template='{{.data.NodeHosts}}' | sed -n -E -e '/[0-9\.]{4,12}\s` + name + `$/!p' -e '$a` + hostsEntry + `' | tr '\n' '^' | busybox xargs -0 printf '{"data": {"NodeHosts":"%s"}}'| sed -E 's%\^%\\n%g') && kubectl patch cm coredns -n kube-system -p="$patch"`
-	successInjectCoreDNSEntry := false
-
-	// try 3 (or K3D_DEBUG_COREDNS_RETRIES value) times, as e.g. on cluster startup it may take some time for the Configmap to be available and the server to be responsive
-	for i := 0; i < retries; i++ {
-		l.Log().Debugf("Running CoreDNS patch in node %s to add %s (try %d/%d)...", node.Name, hostsEntry, i, retries)
-		logreader, err := runtime.ExecInNodeGetLogs(ctx, node, []string{"sh", "-c", patchCmd})
-		if err == nil {
-			successInjectCoreDNSEntry = true
-			break
-		} else {
-			msg := fmt.Sprintf("(try %d/%d) error patching the CoreDNS ConfigMap to include entry '%s': %+v", i, retries, hostsEntry, err)
-			if logreader != nil {
-				readlogs, err := io.ReadAll(logreader)
-				if err != nil {
-					l.Log().Debugf("(try %d/%d) error reading the logs from failed CoreDNS patch exec process in node %s: %v", i, retries, node.Name, err)
-				} else {
-					msg += fmt.Sprintf("\nLogs: %s", string(readlogs))
-				}
-			} else {
-				l.Log().Debugf("(try %d/%d) error reading the logs from failed CoreDNS patch exec process in node %s: no logreader returned for exec process", i, retries, node.Name)
-			}
-			l.Log().Debugln(msg)
-			time.Sleep(1 * time.Second)
-		}
-	}
-	if !successInjectCoreDNSEntry {
-		return fmt.Errorf("failed to patch CoreDNS ConfigMap to include entry '%s' (%d tries, see debug logs)", hostsEntry, retries)
-	}
-	l.Log().Debugf("Successfully patched CoreDNS Configmap with record '%s'", hostsEntry)
-	return nil
-}
-
-// prepInjectHostIP adds /etc/hosts entry for host.k3d.internal, referring to the host system
-func prepInjectHostIP(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Cluster, clusterStartOpts *k3d.ClusterStartOpts) error {
-	if cluster.Network.Name == "host" {
-		l.Log().Debugf("Not injecting host.k3d.internal into /etc/hosts as clusternetwork is 'host'")
-		return nil
-	}
-
-	hostIP := clusterStartOpts.EnvironmentInfo.HostGateway
-	hostsEntry := fmt.Sprintf("%s %s", hostIP.String(), k3d.DefaultK3dInternalHostRecord)
-	l.Log().Infof("Injecting '%s' into /etc/hosts of all nodes...", hostsEntry)
-
-	// entry in /etc/hosts
-	errgrp, errgrpctx := errgroup.WithContext(ctx)
-	for _, node := range cluster.Nodes {
-		n := node
-		errgrp.Go(func() error {
-			return runtime.ExecInNode(errgrpctx, n, []string{"sh", "-c", fmt.Sprintf("echo '%s' >> /etc/hosts", hostsEntry)})
-		})
-	}
-	if err := errgrp.Wait(); err != nil {
-		return fmt.Errorf("failed to add hosts entry %s: %w", hostsEntry, err)
-	}
-	l.Log().Debugf("Successfully added host record \"%s\" to /etc/hosts in all nodes", hostsEntry)
-
-	return nil
-}
-
 func prepCreateLocalRegistryHostingConfigMap(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Cluster) error {
 	success := false
 	for _, node := range cluster.Nodes {
