@@ -1,5 +1,5 @@
 /*
-Copyright © 2020-2021 The k3d Author(s)
+Copyright © 2020-2022 The k3d Author(s)
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -30,7 +30,6 @@ import (
 	"fmt"
 	"io"
 	"os"
-	"reflect"
 	"strconv"
 	"strings"
 	"time"
@@ -96,7 +95,7 @@ func NodeAddToCluster(ctx context.Context, runtime runtimes.Runtime, node *k3d.N
 		l.Log().Debugf("Didn't find node with role '%s' in cluster '%s'. Choosing any other node (and using defaults)...", node.Role, cluster.Name)
 		node.Cmd = k3d.DefaultRoleCmds[node.Role]
 		for _, existingNode := range cluster.Nodes {
-			if existingNode.Role != k3d.LoadBalancerRole { // any role except for the LoadBalancer role
+			if existingNode.Role == k3d.AgentRole || existingNode.Role == k3d.ServerRole { // only K3s nodes
 				srcNode = existingNode
 				break
 			}
@@ -114,12 +113,18 @@ func NodeAddToCluster(ctx context.Context, runtime runtimes.Runtime, node *k3d.N
 	 * -> remove fields that are not safe to copy as they break something down the stream
 	 */
 
+	// sanitize fields that mismatch between roles
+	if srcNode.Role != node.Role {
+		l.Log().Debugf("Dropping some fields from source node because it's not of the same role (%s != %s)...", srcNode.Role, node.Role)
+		srcNode.Memory = "" // memory settings are scoped per role (--servers-memory/--agents-memory)
+	}
+
 	// TODO: I guess proper deduplication can be handled in a cleaner/better way or at the infofaker level at some point
 	for _, forbiddenMount := range util.DoNotCopyVolumeSuffices {
-		for i, mount := range node.Volumes {
+		for i, mount := range srcNode.Volumes {
 			if strings.Contains(mount, forbiddenMount) {
-				l.Log().Tracef("Dropping copied volume mount %s to avoid issues...", mount)
-				node.Volumes = util.RemoveElementFromStringSlice(node.Volumes, i)
+				l.Log().Tracef("Dropping volume mount %s from source node to avoid issues...", mount)
+				srcNode.Volumes = util.RemoveElementFromStringSlice(srcNode.Volumes, i)
 			}
 		}
 	}
@@ -407,9 +412,9 @@ func NodeStart(ctx context.Context, runtime runtimes.Runtime, node *k3d.Node, no
 	// execute lifecycle hook actions
 	for _, hook := range nodeStartOpts.NodeHooks {
 		if hook.Stage == k3d.LifecycleStagePreStart {
-			l.Log().Tracef("Node %s: Executing preStartAction '%s'", node.Name, reflect.TypeOf(hook))
+			l.Log().Tracef("Node %s: Executing preStartAction '%s': %s", node.Name, hook.Action.Name(), hook.Action.Info())
 			if err := hook.Action.Run(ctx, node); err != nil {
-				l.Log().Errorf("Node %s: Failed executing preStartAction '%+v': %+v", node.Name, hook, err)
+				l.Log().Errorf("Node %s: Failed executing preStartAction '%s': %s: %+v", node.Name, hook.Action.Name(), hook.Action.Info(), err) // TODO: should we error out on failed preStartAction?
 			}
 		}
 	}
@@ -424,7 +429,7 @@ func NodeStart(ctx context.Context, runtime runtimes.Runtime, node *k3d.Node, no
 	if node.State.Started != "" {
 		ts, err := time.Parse("2006-01-02T15:04:05.999999999Z", node.State.Started)
 		if err != nil {
-			l.Log().Debugf("Failed to parse '%s.State.Started' timestamp '%s', falling back to calulated time", node.Name, node.State.Started)
+			l.Log().Debugf("Failed to parse '%s.State.Started' timestamp '%s', falling back to calculated time", node.Name, node.State.Started)
 		}
 		startTime = ts.Truncate(time.Second)
 		l.Log().Debugf("Truncated %s to %s", ts, startTime)
