@@ -1,5 +1,5 @@
 /*
-Copyright © 2020-2021 The k3d Author(s)
+Copyright © 2020-2022 The k3d Author(s)
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -23,9 +23,13 @@ THE SOFTWARE.
 package config
 
 import (
-	conf "github.com/rancher/k3d/v5/pkg/config/v1alpha3"
+	"strings"
+
+	conf "github.com/rancher/k3d/v5/pkg/config/v1alpha4"
 	l "github.com/rancher/k3d/v5/pkg/logger"
+	runtimeutil "github.com/rancher/k3d/v5/pkg/runtimes/util"
 	k3d "github.com/rancher/k3d/v5/pkg/types"
+	"github.com/rancher/k3d/v5/pkg/types/k3s"
 )
 
 // ProcessSimpleConfig applies processing to the simple config, sanitizing it and doing some modifications
@@ -56,5 +60,18 @@ func ProcessClusterConfig(clusterConfig conf.ClusterConfig) (*conf.ClusterConfig
 		clusterConfig.ClusterCreateOpts.DisableLoadBalancer = true
 	}
 
+	for _, node := range clusterConfig.Cluster.Nodes {
+		for vIndex, volume := range node.Volumes {
+			_, dest, err := runtimeutil.ReadVolumeMount(volume)
+			if err != nil {
+				return nil, err
+			}
+			if path, ok := k3s.K3sPathShortcuts[dest]; ok {
+				l.Log().Tracef("[node: %s] expanding volume shortcut %s to %s", node.Name, dest, path)
+				node.Volumes[vIndex] = strings.Replace(volume, dest, path, 1)
+			}
+		}
+	}
+
 	return &clusterConfig, nil
 }
