@@ -1,5 +1,5 @@
 /*
-Copyright © 2020-2021 The k3d Author(s)
+Copyright © 2020-2022 The k3d Author(s)
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -32,7 +32,7 @@ import (
 	"github.com/docker/go-connections/nat"
 	cliutil "github.com/rancher/k3d/v5/cmd/util" // TODO: move parseapiport to pkg
 	"github.com/rancher/k3d/v5/pkg/client"
-	conf "github.com/rancher/k3d/v5/pkg/config/v1alpha3"
+	conf "github.com/rancher/k3d/v5/pkg/config/v1alpha4"
 	"github.com/rancher/k3d/v5/pkg/runtimes"
 	k3d "github.com/rancher/k3d/v5/pkg/types"
 	"github.com/rancher/k3d/v5/pkg/types/k3s"
@@ -136,11 +136,12 @@ func TransformSimpleToClusterConfig(ctx context.Context, runtime runtimes.Runtim
 
 	for i := 0; i < simpleConfig.Servers; i++ {
 		serverNode := k3d.Node{
-			Name:       client.GenerateNodeName(newCluster.Name, k3d.ServerRole, i),
-			Role:       k3d.ServerRole,
-			Image:      simpleConfig.Image,
-			ServerOpts: k3d.ServerOpts{},
-			Memory:     simpleConfig.Options.Runtime.ServersMemory,
+			Name:        client.GenerateNodeName(newCluster.Name, k3d.ServerRole, i),
+			Role:        k3d.ServerRole,
+			Image:       simpleConfig.Image,
+			ServerOpts:  k3d.ServerOpts{},
+			Memory:      simpleConfig.Options.Runtime.ServersMemory,
+			HostPidMode: simpleConfig.Options.Runtime.HostPidMode,
 		}
 
 		// first server node will be init node if we have more than one server specified but no external datastore
@@ -158,10 +159,11 @@ func TransformSimpleToClusterConfig(ctx context.Context, runtime runtimes.Runtim
 
 	for i := 0; i < simpleConfig.Agents; i++ {
 		agentNode := k3d.Node{
-			Name:   client.GenerateNodeName(newCluster.Name, k3d.AgentRole, i),
-			Role:   k3d.AgentRole,
-			Image:  simpleConfig.Image,
-			Memory: simpleConfig.Options.Runtime.AgentsMemory,
+			Name:        client.GenerateNodeName(newCluster.Name, k3d.AgentRole, i),
+			Role:        k3d.AgentRole,
+			Image:       simpleConfig.Image,
+			Memory:      simpleConfig.Options.Runtime.AgentsMemory,
+			HostPidMode: simpleConfig.Options.Runtime.HostPidMode,
 		}
 		newCluster.Nodes = append(newCluster.Nodes, &agentNode)
 	}
@@ -277,6 +279,7 @@ func TransformSimpleToClusterConfig(ctx context.Context, runtime runtimes.Runtim
 		GPURequest:          simpleConfig.Options.Runtime.GPURequest,
 		ServersMemory:       simpleConfig.Options.Runtime.ServersMemory,
 		AgentsMemory:        simpleConfig.Options.Runtime.AgentsMemory,
+		HostAliases:         simpleConfig.HostAliases,
 		GlobalLabels:        map[string]string{}, // empty init
 		GlobalEnv:           []string{},          // empty init
 	}
