@@ -1,5 +1,5 @@
 /*
-Copyright © 2020-2021 The k3d Author(s)
+Copyright © 2020-2022 The k3d Author(s)
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,10 +27,11 @@ import (
 	"time"
 
 	k3dc "github.com/rancher/k3d/v5/pkg/client"
-	conf "github.com/rancher/k3d/v5/pkg/config/v1alpha3"
+	conf "github.com/rancher/k3d/v5/pkg/config/v1alpha4"
 	"github.com/rancher/k3d/v5/pkg/runtimes"
 	runtimeutil "github.com/rancher/k3d/v5/pkg/runtimes/util"
 	k3d "github.com/rancher/k3d/v5/pkg/types"
+	"inet.af/netaddr"
 
 	"fmt"
 
@@ -76,13 +77,38 @@ func ValidateClusterConfig(ctx context.Context, runtime runtimes.Runtime, config
 		}
 	}
 
+	// hostAliases
+	if len(config.ClusterCreateOpts.HostAliases) > 0 {
+		// not allowed in hostnetwork mode
+		if config.Cluster.Network.Name == "host" {
+			return fmt.Errorf("hostAliases not allowed in hostnetwork mode")
+		}
+
+		// validate IP and hostname
+		for _, ha := range config.ClusterCreateOpts.HostAliases {
+			// validate IP
+			_, err := netaddr.ParseIP(ha.IP)
+			if err != nil {
+				return fmt.Errorf("invalid IP '%s' in hostAlias '%s': %w", ha.IP, ha, err)
+			}
+
+			// validate hostnames
+			for _, hostname := range ha.Hostnames {
+				if err := k3dc.ValidateHostname(hostname); err != nil {
+					return fmt.Errorf("invalid hostname '%s' in hostAlias '%s': %w", hostname, ha, err)
+				}
+			}
+		}
+
+	}
+
 	// validate nodes one by one
 	for _, node := range config.Cluster.Nodes {
 
 		// volumes have to be either an existing path on the host or a named runtime volume
 		for _, volume := range node.Volumes {
 
-			if err := runtimeutil.ValidateVolumeMount(runtime, volume); err != nil {
+			if err := runtimeutil.ValidateVolumeMount(ctx, runtime, volume, &config.Cluster); err != nil {
 				return fmt.Errorf("failed to validate volume mount '%s': %w", volume, err)
 			}
 		}
